//******************************************************************************
//
// File:    ChatRoom.java
// Package: edu.rit.chat2
// Unit:    Class edu.rit.chat2.ChatRoom
//
// This Java source file is copyright (C) 2006 by Alan Kaminsky. All rights
// reserved. For further information, contact the author, Alan Kaminsky, at
// ark@cs.rit.edu.
//
// This Java source file is part of the M2MI Library ("The Library"). The
// Library is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// The Library is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
// details.
//
// A copy of the GNU General Public License is provided in the file gpl.txt. You
// may also obtain a copy of the GNU General Public License on the World Wide
// Web at http://www.gnu.org/licenses/gpl.html or by writing to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
//******************************************************************************

package edu.rit.chat2;

import edu.rit.m2mi.Multihandle;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

/**
 * Class ChatRoom encapsulates the name and multihandle for a chat room.
 *
 * @author  Alan Kaminsky
 * @version 26-Mar-2006
 */
public class ChatRoom
	implements Externalizable
	{

// Hidden data members.

	private static final long serialVersionUID = -7463719649672679814L;

	private String myName;
	private ChatRef myMultihandle;

// Exported operations.

	/**
	 * Construct a new chat room. This constructor is for use only by object
	 * deserialization.
	 */
	public ChatRoom()
		{
		}

	/**
	 * Construct a new chat room with the given name and multihandle.
	 *
	 * @param  theName         Name.
	 * @param  theMultihandle  Multihandle for interface ChatRef.
	 *
	 * @exception  NullPointerException
	 *     (unchecked exception) Thrown if <TT>theName</TT> is null or
	 *     <TT>theMultihandle</TT> is null.
	 */
	public ChatRoom
		(String theName,
		 ChatRef theMultihandle)
		{
		if (theName == null)
			{
			throw new NullPointerException
				("ChatRoom(): theName is null");
			}
		if (theMultihandle == null)
			{
			throw new NullPointerException
				("ChatRoom(): theMultihandle is null");
			}
		myName = theName;
		myMultihandle = theMultihandle;
		}

// Exported operations.

	/**
	 * Join the given chat object to this chat room.
	 *
	 * @param  theChat  Chat object.
	 */
	public void join
		(Chat theChat)
		{
		((Multihandle) myMultihandle).attach (theChat);
		}

	/**
	 * Disjoin the given chat object from this chat room.
	 *
	 * @param  theChat  Chat object.
	 */
	public void disjoin
		(Chat theChat)
		{
		((Multihandle) myMultihandle).detach (theChat);
		}

	/**
	 * Display the given line of text in the chat log of each chat object that
	 * has joined this chat room.
	 *
	 * @param  line  Line of text.
	 */
	public void putMessage
		(String line)
		{
		myMultihandle.putMessage (line);
		}

	/**
	 * Write this chat room to the given object output stream.
	 *
	 * @param  out  Object output stream.
	 *
	 * @exception  IOException
	 *     Thrown if an I/O error occurred.
	 */
	public void writeExternal
		(ObjectOutput out)
		throws IOException
		{
		out.writeUTF (myName);
		out.writeObject (myMultihandle);
		}

	/**
	 * Read this chat room from the given object input stream.
	 *
	 * @param  in  Object input stream.
	 *
	 * @exception  IOException
	 *     Thrown if an I/O error occurred.
	 * @exception  ClassNotFoundException
	 *     Thrown if a class needed to deserialize this object could not be
	 *     found.
	 */
	public void readExternal
		(ObjectInput in)
		throws IOException, ClassNotFoundException
		{
		myName = in.readUTF();
		myMultihandle = (ChatRef) in.readObject();
		}

	/**
	 * Determine if this chat room is equal to the given object.
	 *
	 * @param  obj  Object to test.
	 *
	 * @return  True if this chat room is equal to <TT>obj</TT>, false
	 *          otherwise.
	 */
	public boolean equals
		(Object obj)
		{
		return
			obj instanceof ChatRoom &&
			this.myName.equals (((ChatRoom) obj).myName) &&
			this.myMultihandle.equals (((ChatRoom) obj).myMultihandle);
		}

	/**
	 * Returns a hash code for this chat room.
	 *
	 * @return  Hash code.
	 */
	public int hashCode()
		{
		return myName.hashCode() + myMultihandle.hashCode();
		}

	/**
	 * Returns a string version of this chat room. The string version is the
	 * chat room's name.
	 *
	 * @return  String version.
	 */
	public String toString()
		{
		return myName;
		}

	}
