//******************************************************************************
//
// File:    SurveyVote.java
// Package: edu.rit.survey
// Unit:    Class edu.rit.survey.SurveyVote
//
// This Java source file is copyright (C) 2006 by Alan Kaminsky. All rights
// reserved. For further information, contact the author, Alan Kaminsky, at
// ark@cs.rit.edu.
//
// This Java source file is part of the M2MI Library ("The Library"). The
// Library is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// The Library is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
// details.
//
// A copy of the GNU General Public License is provided in the file gpl.txt. You
// may also obtain a copy of the GNU General Public License on the World Wide
// Web at http://www.gnu.org/licenses/gpl.html or by writing to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
//******************************************************************************

package edu.rit.survey;

import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

/**
 * Class SurveyVote encapsulates the user's vote in a survey.
 * <P>
 * The survey vote object keeps track of one participant's vote. A timestamp
 * (logical clock value) is associated with the vote as well. Each time the
 * participant changes the vote, the timestamp is incremented. The timestamp is
 * limited to a maximum of 32767.
 *
 * @author  Alan Kaminsky
 * @version 25-Mar-2006
 */
public class SurveyVote
	{

// Hidden data members.

	// The participant's timestamp. If zero, the participant has not voted yet.
	short myTimestamp;

	// The participant's vote.
	byte myVote;

// Exported constructors.

	/**
	 * Construct a new survey vote object. Initially, the participant has not
	 * voted yet.
	 */
	public SurveyVote()
		{
		}

// Exported operations.

	/**
	 * Determine if there is a vote for the given answer.
	 *
	 * @param  i  Answer index (0-3).
	 *
	 * @return  True if there is a vote for answer <TT>i</TT>, false otherwise.
	 */
	public boolean isVoteFor
		(int i)
		{
		return myTimestamp > 0 && myVote == i;
		}

	/**
	 * Record a vote for the given answer. The timestamp is incremented.
	 *
	 * @param  i               Answer index (0-3).
	 *
	 * @exception  IllegalStateException
	 *     (unchecked exception) Thrown if the timestamp is at the maximum
	 *     (32767).
	 */
	public void setVoteFor
		(int i)
		{
		if (myTimestamp == 32767)
			{
			throw new IllegalStateException
				("SurveyVote.setVoteFor(): Timestamp at maximum");
			}
		++ myTimestamp;
		myVote = (byte) i;
		}

	/**
	 * Write this survey vote to the given object output stream.
	 *
	 * @param  out  Object output stream.
	 *
	 * @exception  IOException
	 *     Thrown if an I/O error occurred.
	 */
	public void write
		(ObjectOutput out)
		throws IOException
		{
		out.writeShort (myTimestamp);
		out.writeByte (myVote);
		}

	/**
	 * Read this survey vote from the given object input stream.
	 *
	 * @param  in  Object input stream.
	 *
	 * @exception  IOException
	 *     Thrown if an I/O error occurred.
	 */
	public void read
		(ObjectInput in)
		throws IOException
		{
		myTimestamp = in.readShort();
		myVote = in.readByte();
		}

	}
